use std::{
    cell::{Cell, RefCell},
    collections::{HashMap, HashSet},
    rc::Rc,
};

use crate::fleck_core;

use fleck_core::crypto::PublicKey;

use super::MessageParams;

#[derive(Clone, PartialEq, Debug)]
struct RoutingTable {
    peers: HashMap<PublicKey, HashSet<fleck_core::Peer>>,
}

impl RoutingTable {
    fn update_with(&mut self, other: &Self) {}
}

pub struct PacketRoutingService {}

impl crate::Service for PacketRoutingService {
    fn new(api: std::rc::Rc<crate::API>) -> Self {
        Self {}
    }

    fn setup(self: &std::rc::Rc<Self>) {}
}

struct RoutingTableMessage {
    generation: usize,
    known: Vec<PublicKey>,
}

impl fleck_core::msg::MessageParams for RoutingTableMessage {
    const NAME: &'static str = "RoutingTableMessage";
    const SIGNED: bool = false;
    const ENCRYPTED: bool = true;
}

#[derive(Debug, serde::Serialize, serde::Deserialize)]
struct RoutingTableRequest {
    known_generation: Option<usize>,
}

impl fleck_core::msg::MessageParams for RoutingTableRequest {
    const NAME: &'static str = "RoutingTableRequest";
    const SIGNED: bool = false;
    const ENCRYPTED: bool = true;
}

pub struct RoutingTableService {
    api: Rc<crate::API>,
    table_generation: Cell<usize>,
    table: RefCell<HashMap<PublicKey, HashSet<fleck_core::peer::Peer>>>,
}

impl crate::Service for RoutingTableService {
    fn new(api: std::rc::Rc<crate::API>) -> Self {
        Self {
            api,
            table_generation: Cell::new(0),
            table: Default::default(),
        }
    }

    fn setup(self: &std::rc::Rc<Self>) {
        self.api
            .channel::<fleck_core::NodeRegistrationChannel>()
            .sub_ref(self, Self::new_node);
        self.api
            .channel::<fleck_core::MajorTickChannel>()
            .sub_ref(self, Self::ask_for_updates);
    }
}

impl RoutingTableService {
    fn new_node(&self, node: &mut Rc<fleck_core::Node>) {}

    fn ask_for_updates(&self, _: &mut ()) {
        // TODO: ask each peer for what their known peers are
        // FUTURE TODO: stagger these requests at random offsets
        let peers = self
            .api
            .with_service(|ns: &fleck_core::NodeService| ns.peers());
        for node in peers {
            self.api.queue::<fleck_core::SendPacketChannel>(
                RoutingTableRequest {
                    known_generation: None,
                }
                .into_message()
                .with_node(node),
            );
        }
    }
}
