use std::any::Any;
use std::cell::Ref;

pub trait AsAny {
    fn as_any(&self) -> &dyn Any;
}

impl<T: Any> AsAny for T {
    fn as_any(&self) -> &dyn Any {
        self
    }
}

pub trait IntoWeak<T> {
    fn as_weak(&self) -> std::rc::Weak<T>;
}

impl<T> IntoWeak<T> for std::rc::Rc<T> {
    fn as_weak(&self) -> std::rc::Weak<T> {
        std::rc::Rc::downgrade(self)
    }
}

impl<'a, T> IntoWeak<T> for &'a std::rc::Rc<T> {
    fn as_weak(&self) -> std::rc::Weak<T> {
        std::rc::Rc::downgrade(self)
    }
}

impl<T> IntoWeak<T> for std::rc::Weak<T> {
    fn as_weak(&self) -> std::rc::Weak<T> {
        self.clone()
    }
}

impl<'a, T> IntoWeak<T> for &'a std::rc::Weak<T> {
    fn as_weak(&self) -> std::rc::Weak<T> {
        (*self).clone()
    }
}

pub struct InnerRef<'l, T> {
    r: Ref<'l, Option<T>>,
}

impl<'l, T> InnerRef<'l, T> {
    pub fn build(from: Ref<'l, Option<T>>) -> Option<Self> {
        if from.is_none() {
            None
        } else {
            Some(Self { r: from })
        }
    }
}

impl<'l, T> AsRef<T> for InnerRef<'l, T> {
    fn as_ref(&self) -> &T {
        self.r.as_ref().unwrap()
    }
}

impl<'l, T> std::ops::Deref for InnerRef<'l, T> {
    type Target = T;
    fn deref(&self) -> &Self::Target {
        self.r.as_ref().unwrap()
    }
}
