use crate::fleck_core;

#[test_log::test]
fn end_to_end() {
    let skeleton = super::TestSkeleton::new();

    let span_f1 = tracing::span!(tracing::Level::TRACE, "f1");
    let span_f2 = tracing::span!(tracing::Level::TRACE, "f2");

    let fvec = skeleton.initialize_nodes(2);
    let f1 = fvec[0].clone();
    let f2 = fvec[1].clone();

    // run for a few rounds of the event loops, without firing any ticks.
    for _ in 0..5 {
        {
            let _guard = span_f1.enter();
            f1.run_once();
        }
        {
            let _guard = span_f2.enter();
            f2.run_once();
        }
    }

    let f1pk = f1.with_service(|ns: &fleck_core::NodeService| ns.self_node().pubkey().clone());
    let f2pk = f1.with_service(|ns: &fleck_core::NodeService| ns.self_node().pubkey().clone());

    // check that they know about each other.
    f1.with_service(|ns: &fleck_core::NodeService| {
        assert!(ns.node_by_pubkey(&f2pk).is_some());
    });
    f2.with_service(|ns: &fleck_core::NodeService| {
        assert!(ns.node_by_pubkey(&f1pk).is_some());
    });
}
