use std::{cell::RefCell, rc::Rc};

use crate::fleck_core;

#[derive(Debug, serde::Serialize, serde::Deserialize)]
struct RoutedMessage {
    msg: String,
}

impl fleck_core::MessageParams for RoutedMessage {
    const NAME: &'static str = "RoutedMessage";
    const SIGNED: bool = false;
    const ENCRYPTED: bool = false;
}

#[derive(Default)]
struct ReceiveService {
    messages: RefCell<Vec<String>>,
}

impl crate::DefaultService for ReceiveService {
    fn setup(self: &Rc<Self>, api: Rc<crate::API>) {
        api.channel::<fleck_core::MessageChannel<RoutedMessage>>()
            .sub_eat(self, Self::message);
    }
}

impl ReceiveService {
    fn message(&self, msg: (fleck_core::msg::Metadata, RoutedMessage)) {
        self.messages.borrow_mut().push(msg.1.msg);
    }
}

#[test_log::test]
fn one_way_test() {
    let skeleton = super::TestSkeleton::new();

    let span_f1 = tracing::span!(tracing::Level::TRACE, "f1");
    let span_f2 = tracing::span!(tracing::Level::TRACE, "f2");
    let span_f3 = tracing::span!(tracing::Level::TRACE, "f3");

    let fvec = skeleton.initialize_nodes(3);
    let f1 = fvec[0].clone();
    let f2 = fvec[1].clone();
    let f3 = fvec[2].clone();

    // we explicitly *don't* tell f1 about the message type, because it should never see it anyway
    f2.with_service(|ms: &fleck_core::MessageService| {
        ms.add_message_type::<RoutedMessage>();
    });
    f3.with_service(|ms: &fleck_core::MessageService| {
        ms.add_message_type::<RoutedMessage>();
    });

    f2.add_service::<ReceiveService>();
    f3.add_service::<ReceiveService>();

    // let the encryption handshake and routing table exchange happen
    for _ in 0..5 {
        {
            let _guard = span_f1.enter();
            f1.run_once();
        }
        {
            let _guard = span_f2.enter();
            f2.run_once();
        }
        {
            let _guard = span_f3.enter();
            f2.run_once();
        }
    }
}
